package net.wanji.opt.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.databus.bo.CrossIdAndStartEndDateBO;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dto.MetricHistDTO;
import net.wanji.opt.service.SceneEvaluateService;
import net.wanji.opt.vo.RunningEvaluateMetricsDetailVO;
import net.wanji.opt.vo.SceneEvaluateabnormalDistributeVO;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/8/28 16:38
 */
@Slf4j
@Service
public class SceneEvaluateServiceImpl implements SceneEvaluateService {
    private final CrossDataHistMapper crossDataHistMapper;
    private final RunningEvaluateServiceImpl runningEvaluateService;

    public SceneEvaluateServiceImpl(CrossDataHistMapper crossDataHistMapper, RunningEvaluateServiceImpl runningEvaluateService) {
        this.crossDataHistMapper = crossDataHistMapper;
        this.runningEvaluateService = runningEvaluateService;
    }

    @Override
    public SceneEvaluateabnormalDistributeVO abnormalDistribute(CrossIdAndStartEndDateBO bo) {
        String crossId = bo.getCrossId();
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();

        SceneEvaluateabnormalDistributeVO vo = new SceneEvaluateabnormalDistributeVO();
        List<SceneEvaluateabnormalDistributeVO.TimeDistribution> timeDistributionList =
                buildTimeDistributionList(crossId, startDate, endDate);
        vo.setTimeDistributionList(timeDistributionList);

        setCounts(timeDistributionList, vo);
        return vo;
    }

    private static void setCounts(List<SceneEvaluateabnormalDistributeVO.TimeDistribution> timeDistributionList,
                                  SceneEvaluateabnormalDistributeVO vo) {
        int congestionDuration = 0;
        int congestionTimes = 0;
        int unbalanceDuration = 0;
        int unbalanceTimes = 0;
        int spilloverDuration = 0;
        int spilloverTimes = 0;
        for (SceneEvaluateabnormalDistributeVO.TimeDistribution timeDistribution : timeDistributionList) {
            List<RunningEvaluateMetricsDetailVO.ProblemStatus> problemStatusList =
                    timeDistribution.getProblemStatusList();
            for (RunningEvaluateMetricsDetailVO.ProblemStatus problemStatus : problemStatusList) {
                Integer status = problemStatus.getStatus();
                Integer durationMinutes = problemStatus.getDurationMinutes();
                if (status != null && status.equals(CrossStatusEnum.CONGESTION.getCode())) {
                    congestionTimes++;
                    congestionDuration += durationMinutes;
                } else if (status != null && status.equals(CrossStatusEnum.UNBALANCE.getCode())) {
                    unbalanceTimes++;
                    unbalanceDuration += durationMinutes;
                } else if (status != null && status.equals(CrossStatusEnum.SPILLOVER.getCode())) {
                    spilloverTimes++;
                    spilloverDuration += durationMinutes;
                }
            }
        }
        vo.setCongestionTimes(congestionTimes);
        vo.setUnbalanceTimes(unbalanceTimes);
        vo.setSpilloverTimes(spilloverTimes);
        vo.setCongestionDuration(congestionDuration);
        vo.setUnbalanceDuration(unbalanceDuration);
        vo.setSpilloverDuration(spilloverDuration);
    }

    private List<SceneEvaluateabnormalDistributeVO.TimeDistribution> buildTimeDistributionList(
            String crossId, Date startDate, Date endDate) {
        List<SceneEvaluateabnormalDistributeVO.TimeDistribution> res = new ArrayList<>();
        Calendar start = Calendar.getInstance();
        Calendar end = Calendar.getInstance();
        start.setTime(startDate);
        end.setTime(endDate);
        // 使 endDate 包含在内
        end.add(Calendar.DATE, 1);

        for (Date date = start.getTime(); start.before(end); start.add(Calendar.DATE, 1), date = start.getTime()) {
            SceneEvaluateabnormalDistributeVO.TimeDistribution timeDistribution =
                    new SceneEvaluateabnormalDistributeVO.TimeDistribution();
            timeDistribution.setProblemDate(date);
            // 获取问题列表
            // 路口级别全量数据
            int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
            int endStamp = (int) (endDate.getTime() / 1000);
            List<MetricHistDTO> crossDTOList = crossDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
            // 过滤有问题的记录
            List<MetricHistDTO> filteredList = crossDTOList.stream()
                    .filter(metricHistDTO -> metricHistDTO.getStatus() != null && !metricHistDTO.getStatus().equals(0))
                    .collect(Collectors.toList());
            timeDistribution.setProblemStatusList(runningEvaluateService.buildProblemStatusList(filteredList));
            res.add(timeDistribution);
        }
        return res;
    }
}
